package eu.unitn.disi.db.dkcore;

/**
 * Implementation of "Distributed  k-Core View Materialization and Maintenance for Large Dynamic Graphs"
 *  from Aksu, H. Dept. of Comput. Eng., Bilkent Univ., Ankara, Turkey
 *  Canim, M. ; Yuan-Chi Chang ; Korpeoglu, I. ; Ulusoy, O.
 *
 * @author Martin Brugnara (martin.brugnara@alumni.unitn.it) [<a href="http://db.disi.unitn.eu/">dbTrento</a>]
 */

import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.util.Bytes;


public class Config {

    // Singleton structure
    private static Config self = null;

    private Config(){
        super();
        setUpConf();
    }

    public static Config getConfig(){
        if (self == null) {
            synchronized (Configuration.class) {
                if (self == null) {
                    self = new Config();
                }
            }
        }
        return self;
    }


    // Data
    final String ZK_QUORUM = "127.0.0.1";
    final String CLIENT_PORT = "2181"; // default: 2181

    public Configuration hbaseConf = null;
    private void setUpConf() {
        hbaseConf = HBaseConfiguration.create();
        hbaseConf.set("hbase.zookeeper.quorum", ZK_QUORUM);
        hbaseConf.set("hbase.zookeeper.property.clientPort", CLIENT_PORT);
        hbaseConf.setLong("hbase.rpc.timeout", RPC_TIMEOUT_MINUTES * 60 * 1000);
    }

    private static final int RPC_TIMEOUT_MINUTES = 120; // 2 hours
    protected static final long INSERT_TIMEOUT = 10 * 60 * 1000; // 10 min

    // Constant
    public static final String GRAPH_TABLE_NAME = "graph";
    public static final String TL_TABLE_NAME = "tl";

    public final static int NODE_ENCODING_LENGTH = 10;
    public final static byte[] EMPTY_FAMILY = Bytes.toBytes("empty_meta");
    public final static byte[] EMPTY = new byte[0];

    public final static byte[] QNCS = Bytes.toBytes("qncs");
    public final static byte[] DEGREES = Bytes.toBytes("degrees");

    public final static int PUT_BATCH_SIZE = 200;


    // IMPORTANT: tweak this parameters
    public static final int SCANNER_CACHE = 300;
    public static final int PLAUSIBLE_DEGREE = 50;
    public static final int SCANNER_BATCH = 100;
    // 50 000 * (10 char) = 500 000 =+- 0.5M
    public static final int DEGREE_BULK_GET_LIMIT = 50000;
}
