package eu.unitn.disi.db.dkcore;

/**
 * Implementation of "Distributed  k-Core View Materialization and Maintenance for Large Dynamic Graphs"
 *  from Aksu, H. Dept. of Comput. Eng., Bilkent Univ., Ankara, Turkey
 *  Canim, M. ; Yuan-Chi Chang ; Korpeoglu, I. ; Ulusoy, O.
 *
 * @author Martin Brugnara (martin.brugnara@alumni.unitn.it) [<a href="http://db.disi.unitn.eu/">dbTrento</a>]
 */

import org.apache.hadoop.hbase.client.*;
import org.apache.hadoop.hbase.filter.FilterList;
import org.apache.hadoop.hbase.filter.FirstKeyOnlyFilter;
import org.apache.hadoop.hbase.filter.KeyOnlyFilter;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;

import static eu.unitn.disi.db.dkcore.Config.NODE_ENCODING_LENGTH;
import static eu.unitn.disi.db.dkcore.Config.SCANNER_CACHE;
import static org.apache.hadoop.hbase.filter.FilterList.Operator.MUST_PASS_ALL;

public class Dump {

    public static void dump(int k) {
        try {
            BufferedWriter out = new BufferedWriter(new OutputStreamWriter(System.out));

            Config config = Config.getConfig();
            HConnection connection;
            connection = HConnectionManager.createConnection(config.hbaseConf);
            HTableInterface kgraph = connection.getTable(Utils.kGraphName(k));

            Scan scan = new Scan();
            scan.setFilter(new FilterList(MUST_PASS_ALL, new FirstKeyOnlyFilter(), new KeyOnlyFilter()));
            scan.setCaching(SCANNER_CACHE);
            ResultScanner scanner = kgraph.getScanner(scan);

            String currentNode = "";
            Result r;
            while ((r = scanner.next()) != null) {
                String edge = new String(r.getRow());
                if (!currentNode.regionMatches(0, edge, 0, NODE_ENCODING_LENGTH)) {
                    currentNode = edge.substring(0, NODE_ENCODING_LENGTH);
                    out.write(String.valueOf(Long.parseLong(currentNode, 10)));
                    out.newLine();
                }
            }
            out.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 0) k to be dumped
     */
    public static void main(String ... args){
        if (args.length != 1) {
            System.out.println("Args: K");
            System.exit(1);
        }

        try {
            Dump.dump(Integer.parseInt(args[0]));
        } catch (NumberFormatException e) {
            System.err.printf("%s is not a valid value for k\n", args[0]);
            System.exit(1);
        }
    }
}
