package eu.unitn.disi.db.dkcore;

/**
 * Implementation of "Distributed  k-Core View Materialization and Maintenance for Large Dynamic Graphs"
 *  from Aksu, H. Dept. of Comput. Eng., Bilkent Univ., Ankara, Turkey
 *  Canim, M. ; Yuan-Chi Chang ; Korpeoglu, I. ; Ulusoy, O.
 *
 * @author Martin Brugnara (martin.brugnara@alumni.unitn.it) [<a href="http://db.disi.unitn.eu/">dbTrento</a>]
 */

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.hbase.HTableDescriptor;
import org.apache.hadoop.hbase.client.HBaseAdmin;
import org.apache.hadoop.hbase.client.HConnection;
import org.apache.hadoop.hbase.client.HConnectionManager;
import org.apache.hadoop.hbase.regionserver.KeyPrefixRegionSplitPolicy;
import org.apache.hadoop.hbase.util.Bytes;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

public class Shared {
    private static final Log LOG = LogFactory.getLog(Shared.class);

    // return true on success false otherwise
    public static boolean createTable(HTableDescriptor tableDescriptor, byte[][] splitKeys) {
        Config config = Config.getConfig();
        try {
            LOG.info("add defaults to table desc");
            tableDescriptor.setValue(HTableDescriptor.SPLIT_POLICY, KeyPrefixRegionSplitPolicy.class.getName());
            tableDescriptor.setValue("prefix_split_key_policy.prefix_length", String.valueOf(Config.NODE_ENCODING_LENGTH));

            LOG.info("get connection");
            HConnection connection = HConnectionManager.createConnection(config.hbaseConf);
            LOG.info("got connection");

            if (connection.isAborted() || connection.isClosed() || !connection.isMasterRunning())
                throw new IOException("WTF");

            LOG.info("get admin");
            HBaseAdmin admin = new HBaseAdmin(connection);

            LOG.info("create table");
            if (admin.tableExists(tableDescriptor.getName())) {
                LOG.error(String.format("Table yet exists: %s", tableDescriptor.getNameAsString()));
                return false;
            }
            admin.createTable(tableDescriptor, splitKeys);
            admin.close();

            connection.close();

            return true;

        } catch (IOException e) {
            e.printStackTrace();
        }

        return false;
    }

    public static byte[][] readSplitKeys(InputStream src) throws IOException {
        ArrayList<String> keys = new ArrayList<>();

        BufferedReader br = new BufferedReader(new InputStreamReader(src));
        String input;
        while ((input = br.readLine()) != null)
            keys.add(Utils.pad(input.trim()));

        byte ret[][] = new byte[keys.size()][];
        for (int i=0; i<keys.size(); i++)
            ret[i] = Bytes.toBytes(keys.get(i));

        return ret;
    }
}
